const express = require('express');
require('express-async-errors');
const dotenv =require('dotenv');
const cors = require('cors');
const morgan = require('morgan');
const helmet = require("helmet");
const rateLimiter = require('express-rate-limit');
const {handleError} = require('./utils/handle-errors');
const mongoose = require("mongoose");
//Modele mongodbAtlas:
const User = require('./models/User.js');
const Message = require('./models/Message.js');

const loginRouter = require("./routes/auth-login");
const registerRouter = require("./routes/auth-register");
//Ruty weryfikacji :



const app = express();

dotenv.config({path: '.env'});


const connection = async()=> {
    await mongoose.connect(process.env.MONGO_URL,  () => {

        console.log('MONGO DB IS CONNECTED');
    });

}

connection().catch(err => console.log(err));

const urlix = 'https://mumiecki.networkmanager.pl/';
app.use(cors({

 origin:'https://mumiecki.networkmanager.pl'

}
));
app.use(express.json());
app.use(morgan('common'));
app.use(helmet());
app.use(express.urlencoded({extended: true}));
app.use(rateLimiter({
    windowMs: 5 * 60 * 1000,
    max: 100,
    standardHeaders: true,
    legacyHeaders: false,
}));

// socket io utworzenie pokojów;
const rooms = ['Ogólny', 'Sport', 'Informatyka', 'Kryptowaluty'];
const server = require('http').createServer(app);
const PORT = 3002;
const io = require('socket.io')(server, {

    cors: {
        origin: urlix,
        methods: ['GET', 'POST']
    }

});


app.get('/api/rooms', (req, res) => {
    res.json(rooms);

})

// Dodanie Daty do wiadomosci:
const lastMessage = async (room) => {
    let roomMessages = await Message.aggregate([
        {$match: {to: room}},
        {$group: {_id: '$date', messagesByDate: {$push: '$$ROOT'}}}

    ])
    return roomMessages;


}

const sortMessagesByDate = (messages) => {

    return messages.sort((a, b) => {

        let date1 = a._id.split('/');
        let date2 = b._id.split('/');

        date1 = date1[2] + date1[0] + date1[1];
        date2 = date2[2] + date2[0] + date2[1];

        return date1 < date2 ? -1 : 1;


    })

}


// socket.io nowy uzytkownik , dolaczenie do pokoju rozmow.
io.on('connection', (socket) => {

    socket.on('new-user', async () => {
        const members = await User.find();
        io.emit('new-user', members)

    })

    socket.on('join-room', async (room) => {

        socket.join(room);
        let roomMessages = await lastMessage(room);
        roomMessages = sortMessagesByDate(roomMessages);
        socket.emit('room-messages', roomMessages)


    })

    socket.on('message-room', async (room, content, sender , time, date ) => {

        const newMessage = await Message.create({content, from: sender, time, date, to: room});
        let roomMessages = await lastMessage(room);
        roomMessages = sortMessagesByDate(roomMessages);

        //Wysylanie wiadomosci do pokoju :

        io.to(room).emit('room-messages', roomMessages);

        socket.broadcast.emit('notifications', room)


    })

// Sesja do wylogowywania :
    app.delete('/api/logout', async (req, res) => {

        try {

            const {_id, newMessages} = req.body;
            const user = await User.findById(_id);
            user.status = "offline";
            user.newMessages = newMessages;
            await user.save();
            const members = await User.find();
            socket.broadcast.emit('new-user', members);
            res.status(200).send();


        } catch (e) {

            console.log(e);
            res
                .status(400)
                .send();

        }


    })


})


app.use('/api/auth/login', loginRouter);
app.use('/api/auth/register', registerRouter);
//app.use(handleError);


server.listen(PORT, '0.0.0.0', () => {
    console.log('listening to port ', PORT);

});

